// @name:   Web Enabled TV Remote Sketch
// @author: falldeaf@gmail.com
// @date:   Jan 31, 2012
// @source: For more information visit - http://falldeaf.com

//#include "WProgram.h"
#include <Wire.h>
#include <E24C1024.h>
#include <EDB.h>
#include <EEPROM.h>
#include <IRremote.h>
#include <RgbLed.h>
#include <SPI.h>
#include <Ethernet.h>
//#include <avr/pgmspace.h>

// Enter a MAC address and IP address for your controller below.
// The IP address will be dependent on your local network:
byte mac[] = { 
  0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };
byte ip[] = { 
  192,168,1,177 };
Server server(80);

byte RECV_PIN = 7;

byte knockSensor = 3;
byte statePin = LOW;
byte THRESHOLD = 100;
byte val = 0;

#define LED1_PIN_RED   6
#define LED1_PIN_BLUE  5
#define LED1_PIN_GREEN 9
RgbLedCommonCathode led_rgb1(LED1_PIN_RED, LED1_PIN_BLUE, LED1_PIN_GREEN, true);
byte common_color[3] = {
  0, 0, 255};
byte yes_color[3] = {
  0, 255, 0};
byte no_color[3] = {
  255, 0, 0};

boolean IR_receive_flag = 0;
byte knock = 0;

IRrecv irrecv(RECV_PIN);
IRsend irsend;

decode_results results;

char buffer[30];
int  count = 0;

#define TABLE_SIZE 131072 // 1 device

// IR Code database definitions
struct LogIRCode {
  int id;
  char ident1;
  char ident2;
  char ident3;
  char ident4;
  char ident5;  
  int codeType; // The type of code
  unsigned long codeValue; // The code value if not raw
  unsigned int rawCodes[RAWBUF]; // The durations if raw
  int codeLen; // The length of the code
  int toggle; // The RC5/6 toggle state 
} 
logIRCode;

// The read and write handlers for using the EEPROM Library
void writer(unsigned long address, byte data)
{
  EEPROM1024.write(address, data);
}

byte reader(unsigned long address)
{
  return EEPROM1024.read(address);
}

// Create an EDB object with the appropriate write and read handlers
EDB db(&writer, &reader);

void setup()
{
  //Serial.begin(9600);
  db.open(50);
  //db.create(0, TABLE_SIZE, (unsigned int)sizeof(logIRCode));
  //Serial.print("rc:"); 
  //Serial.println(db.count());

  //deleteAll();

  //if(db.count() < 1)
  //{   
  //  db.create(50, TABLE_SIZE, (unsigned int)sizeof(logIRCode));
  //}

  //Remember common color  
  common_color[0] = reader( 1 );
  common_color[1] = reader( 2 );
  common_color[2] = reader( 3 );
  knock = reader( 4 );

  irrecv.enableIRIn(); // Start the receiver
  led_rgb1.setColor(common_color[0], common_color[1], common_color[2]);
  Ethernet.begin(mac, ip);
  server.begin();
}

// Temporary storage for the recorded code
char ident1;
char ident2;
char ident3;
char ident4;
char ident5;  
int codeType = -1; // The type of code
unsigned long codeValue; // The code value if not raw
unsigned int rawCodes[RAWBUF]; // The durations if raw
int codeLen; // The length of the code
int toggle = 0; // The RC5/6 toggle state


// Stores the code for later playback
// Most of this code is just logging
void storeCode(decode_results *results) {
  codeType = results->decode_type;
  int count = results->rawlen;
  if (codeType == UNKNOWN) {
    //Serial.println("Received unknown code, saving as raw");
    codeLen = results->rawlen - 1;
    // To store raw codes:
    // Drop first value (gap)
    // Convert from ticks to microseconds
    // Tweak marks shorter, and spaces longer to cancel out IR receiver distortion
    for (int i = 1; i <= codeLen; i++) {
      if (i % 2) {
        // Mark
        rawCodes[i - 1] = results->rawbuf[i]*USECPERTICK - MARK_EXCESS;
        //Serial.print(" m");
      } 
      else {
        // Space
        rawCodes[i - 1] = results->rawbuf[i]*USECPERTICK + MARK_EXCESS;
        //Serial.print(" s");
      }
      //Serial.print(rawCodes[i - 1], DEC);
    }
    //Serial.println("");
  }
  else {
    if (codeType == NEC) {
      //Serial.print("Received NEC: ");
      if (results->value == REPEAT) {
        // Don't record a NEC repeat value as that's useless.
        //Serial.println("repeat; ignoring.");
        return;
      }
    } 
    else if (codeType == SONY) {
      //Serial.print("Received SONY: ");
    } 
    else if (codeType == RC5) {
      //Serial.print("Received RC5: ");
    } 
    else if (codeType == RC6) {
      //Serial.print("Received RC6: ");
    } 
    else {
      //Serial.print("Unexpected codeType ");
      //Serial.print(codeType, DEC);
      //Serial.println("");
    }
    Serial.println(results->value, HEX);
    codeValue = results->value;
    codeLen = results->bits;
  }
}

void sendCode(int repeat) {
  if (codeType == NEC) {
    if (repeat) {
      irsend.sendNEC(REPEAT, codeLen);
      //Serial.println("Sent NEC repeat");
    } 
    else {
      irsend.sendNEC(codeValue, codeLen);
      //Serial.print("Sent NEC ");
      //Serial.println(codeValue, HEX);
    }
  } 
  else if (codeType == SONY) {
    irsend.sendSony(codeValue, codeLen);
    //Serial.print("Sent Sony ");
    //Serial.println(codeValue, HEX);
  } 
  else if (codeType == RC5 || codeType == RC6) {
    if (!repeat) {
      // Flip the toggle bit for a new button press
      toggle = 1 - toggle;
    }
    // Put the toggle bit into the code to send
    codeValue = codeValue & ~(1 << (codeLen - 1));
    codeValue = codeValue | (toggle << (codeLen - 1));
    if (codeType == RC5) {
      //Serial.print("Sent RC5 ");
      //Serial.println(codeValue, HEX);
      irsend.sendRC5(codeValue, codeLen);
    } 
    else {
      irsend.sendRC6(codeValue, codeLen);
      //Serial.print("Sent RC6 ");
      //Serial.println(codeValue, HEX);
    }
  } 
  else if (codeType == UNKNOWN) {
    // Assume 38 KHz
    irsend.sendRaw(rawCodes, codeLen, 38);
    //Serial.println("Sent raw");
  }
}

void sendCodeTimes(int repeat)
{
  led_rgb1.setColor(yes_color[0], yes_color[1], yes_color[2]);    
  for(byte k=0; k <= repeat; k++)
  {
    sendCode(1);
    delay(10);
  }

  irrecv.enableIRIn(); // Start the receiver
  //irrecv.resume(); // resume receiver

  led_rgb1.setColor(common_color[0], common_color[1], common_color[2]);    
}

//int lastButtonState;

// utility functions

//void recordLimit()
//{
//Serial.print("Record Limit: ");
//Serial.println(db.limit());
//}

void deleteOneRecord(int recno)
{
  //Serial.print("Deleting recno: ");
  //Serial.println(recno);
  db.deleteRec(recno);
}

void deleteAll()
{
  led_rgb1.setColor(no_color[0], no_color[1], no_color[2]);    
  //Serial.print("Truncating table...");
  db.clear();
  //Serial.println("DONE");
  led_rgb1.setColor(common_color[0], common_color[1], common_color[2]);    
}

//void countRecords()
//{
//Serial.print("Record Count: "); 
//Serial.println(db.count());
//}

boolean isBufferInDB()
{  
  for (int recno = 1; recno <= db.count(); recno++)
  {
    EDB_Status result = db.readRec(recno, EDB_REC logIRCode);
    if (result == EDB_OK)
    {
      if(logIRCode.codeType == codeType && 
        logIRCode.codeValue == codeValue && 
        logIRCode.rawCodes[RAWBUF] == rawCodes[RAWBUF] && 
        logIRCode.codeLen == codeLen &&
        logIRCode.toggle == toggle)
      {
        //Serial.println("I already got that one!"); 
        return true;
      }
    }
    //else printError(result);
  }
  return false;
}

void selectAllWeb(Client client)
{  

  for (int recno = 2; recno <= db.count(); recno++)
  {
    EDB_Status result = db.readRec(recno, EDB_REC logIRCode);
    if (result == EDB_OK)
    {
      client.print(recno);
      client.print(",");

      if(logIRCode.ident1 == 'n' && logIRCode.ident2 == 'u' && logIRCode.ident3 == 'l' && logIRCode.ident4 == 'l')
      {
        client.print(recno);
      } 
      else {
        client.print(logIRCode.ident1);
        client.print(logIRCode.ident2);
        client.print(logIRCode.ident3);
        client.print(logIRCode.ident4);
        client.print(logIRCode.ident5);      
      }

      client.print("|");

      /*
        client.println(); 
       
       client.println();
       client.print("<!-- Record Number:");
       client.print(recno);
       client.print(" | Ident: "); 
       client.print(logIRCode.ident1);
       client.print(logIRCode.ident2);
       client.print(logIRCode.ident3);
       client.print(logIRCode.ident4);
       client.print(logIRCode.ident5);      
       client.print(" | Knock: "); 
       client.print(logIRCode.knock);      
       client.print(" | Extended: "); 
       client.println(logIRCode.extended);
       
       client.print(" | codetype: "); 
       client.print(logIRCode.codeType);   
       client.print(" | codevalue: "); 
       client.print(logIRCode.codeValue);   
       client.print(" | rawCodes: "); 
       client.print(logIRCode.rawCodes[RAWBUF]);   
       client.print(" | codeLen: "); 
       client.print(logIRCode.codeLen);   
       client.print(" | toggle: "); 
       client.println(logIRCode.toggle);
       client.println("-->"); 
       client.println();
       */


    }
    //else printError(result);
  }
}

void updateOneRecord(int recno)
{
  //Serial.print("Updating record at recno: ");
  //Serial.print(recno);
  //Serial.print("...");
  logIRCode.ident1 = ident1;
  logIRCode.ident2 = ident2;
  logIRCode.ident3 = ident3;
  logIRCode.ident4 = ident4;
  logIRCode.ident5 = ident5;  
  logIRCode.codeType = codeType;
  logIRCode.codeValue = codeValue;
  logIRCode.rawCodes[RAWBUF] = rawCodes[RAWBUF];
  logIRCode.codeLen = codeLen;
  logIRCode.toggle = toggle;
  EDB_Status result = db.updateRec(recno, EDB_REC logIRCode);
  //if (result != EDB_OK) printError(result);
  //Serial.println("DONE");
}

void changeLabel(int recno)
{
  getOneRecord(recno);
  updateOneRecord(recno);
}


void appendOneRecord()
{
  logIRCode.ident1 = 'n';
  logIRCode.ident2 = 'u';
  logIRCode.ident3 = 'l';
  logIRCode.ident4 = 'l';
  logIRCode.ident5 = 'y';  
  logIRCode.codeType = codeType;
  logIRCode.codeValue = codeValue;
  logIRCode.rawCodes[RAWBUF] = rawCodes[RAWBUF];
  logIRCode.codeLen = codeLen;
  logIRCode.toggle = toggle;
  EDB_Status result = db.appendRec(EDB_REC logIRCode);
  //if (result != EDB_OK) printError(result);
}

void getOneRecord(int recno)
{
  EDB_Status result = db.readRec(recno, EDB_REC logIRCode);
  if (result == EDB_OK)
  {
    codeType = logIRCode.codeType;
    codeValue = logIRCode.codeValue;
    rawCodes[RAWBUF] = logIRCode.rawCodes[RAWBUF];
    codeLen = logIRCode.codeLen;
    toggle = logIRCode.toggle;
  }
  //else printError(result);
}

//void printError(EDB_Status err)
//{
/*
  //Serial.print("ERROR: ");
 switch (err)
 {
 case EDB_OUT_OF_RANGE:
 //Serial.println("Recno out of range");
 break;
 case EDB_TABLE_FULL:
 //Serial.println("Table full");
 break;
 case EDB_OK:
 default:
 //Serial.println("OK");
 break;
 }
 error_led();
 */
//}

void error_led()
{
  for(int i = 0; i <= 4; i++)
  {
    led_rgb1.setColor(no_color[0], no_color[1], no_color[2]); 
    delay(100);  
    led_rgb1.setColor(common_color[0], common_color[1], common_color[2]);    
    delay(50);
  }
}

// Reads a 2 byte HEX from the serial port and converts it to DEC
// The incomming data stream is build up as follows
// Example:  15	06	A3	FF	00	... etc.
int Read2HEXtoDEC(char Pos1, char Pos2) {    
  // Now convert the HEX to DEC

  int DECval = 0;

  // Convert Pos2 from 16 base to 10 base
  if(Pos2 <= 57) {
    DECval = DECval + (Pos2-48);
  } 
  else {
    DECval = DECval + (Pos2-55);
  }

  // Convert Pos1 from 16 base to 10 base
  if(Pos1 <= 57) {
    DECval = DECval + 16*(Pos1-48);
  } 
  else {
    DECval = DECval +  16*(Pos1-55);
  }

  return DECval;
}

void http_response(Client client)
{
  client.println("HTTP/1.1 200 OK");
  client.println("Content-Type: text/html");
  client.println();
}

void loop() {
  val = analogRead(knockSensor);

  if (val >= THRESHOLD) {
    //Serial.println("Knock!");
    getOneRecord( knock );
    sendCodeTimes(4);
    //selectAll();
  }

  if (irrecv.decode(&results)) {
    led_rgb1.setColor(no_color[0], no_color[1], no_color[2]);
    //digitalWrite(STATUS_PIN, HIGH);
    storeCode(&results);
    if(isBufferInDB())
    {
      error_led();
    } 
    else {
      appendOneRecord();
    }

    irrecv.resume(); // resume receiver
    //digitalWrite(STATUS_PIN, LOW);
    delay(300);
    led_rgb1.setColor(common_color[0], common_color[1], common_color[2]);
  }


  // listen for incoming clients
  Client client = server.available();
  if (client) {
    // an http request ends with a blank line
    boolean currentLineIsBlank = true;
    while (client.connected()) {
      if (client.available()) {
        char c = client.read();

        if(count >= 5 && count <= 16) { 
          buffer[count] = c; 
        }

        // if you've gotten to the end of the line (received a newline
        // character) and the line is blank, the http request has ended,
        // so you can send a reply
        if (c == '\n' && currentLineIsBlank) {

          //Set (K)nock number//
          if(buffer[5] == 'k') {

            //http_response(client);
            String recno_string = String( String(buffer[6]) + String(buffer[7]) + String(buffer[8]) + String(buffer[9]) + String(buffer[10]) + String(buffer[11]) + "\n");

            byte recno_final;
            char carray[7];
            recno_string.toCharArray(carray, sizeof(carray));
            recno_final = atoi(carray);

            writer(4, recno_final );
            knock = recno_final;

            http_response(client);
            client.println(recno_final);

            //Change button (L)abel//
          } 
          else if(buffer[5] == 'l') {
            String recno_string = String( String(buffer[6]) + String(buffer[7]) + String(buffer[8]) + String(buffer[9]) + String(buffer[10]) + String(buffer[11]) + "\n");

            int recno_final;
            char carray[7];
            recno_string.toCharArray(carray, sizeof(carray));
            recno_final = atoi(carray);

            ident1 = buffer[12];
            ident2 = buffer[13];
            ident3 = buffer[14];
            ident4 = buffer[15];
            ident5 = buffer[16];
            changeLabel(recno_final);

            http_response(client);
            client.print(recno_final);                            
            //Toggle (R)eceive mode// not yet implemented //  
            /*} else if(buffer[5] == 'r') {
             
             if(buffer[6] == '1') {
             if(IR_receive_flag == true)
             {
             IR_receive_flag == false;
             } else {
             IR_receive_flag == true;
             }
             }
             
             http_response(client);
             client.print(IR_receive_flag);*/

            //(S)end IR code//  
          } 
          else if(buffer[5] == 's') {

            String recno_string = String( String(buffer[6]) + String(buffer[7]) + String(buffer[8]) + String(buffer[9]) + String(buffer[10]) + String(buffer[11]) + "\n");

            byte recno_final;
            char carray[7];
            recno_string.toCharArray(carray, sizeof(carray));
            recno_final = atoi(carray);

            getOneRecord(recno_final);
            sendCodeTimes(4);

            http_response(client);
            client.println(recno_final);

            //Select (A)ll from database  
          } 
          else if(buffer[5] == 'a') {

            http_response(client);

            selectAllWeb(client);

            //(D)elete database entry//  
          } 
          else if(buffer[5] == 'd') {

            //http_response(client);
            String recno_string = String( String(buffer[6]) + String(buffer[7]) + String(buffer[8]) + String(buffer[9]) + String(buffer[10]) + String(buffer[11]) + "\n");

            byte recno_final;
            char carray[7];
            recno_string.toCharArray(carray, sizeof(carray));
            recno_final = atoi(carray);

            //client.println();

            deleteOneRecord(recno_final);

            http_response(client);
            client.println(recno_final);

            //(W)ipe out entire database//
          } 
          else if(buffer[5] == 'w') {

            deleteAll();
            http_response(client);
            client.println(1);

            //Receive new (C)olor value//
          } 
          else if(buffer[5] == 'c') {

            writer(1, Read2HEXtoDEC( buffer[6], buffer[7]) );
            writer(2, Read2HEXtoDEC( buffer[8], buffer[9]) );
            writer(3, Read2HEXtoDEC( buffer[10], buffer[11]) );

            common_color[0] = Read2HEXtoDEC(buffer[6], buffer[7]);
            common_color[1] = Read2HEXtoDEC(buffer[8], buffer[9]);
            common_color[2] = Read2HEXtoDEC(buffer[10], buffer[11]);              
            led_rgb1.setColor(common_color[0], common_color[1], common_color[2]);

            //Send current color (V)alue//
          } 
          else if(buffer[5] == 'v') {

            http_response(client);
            client.print( common_color[0], DEC );
            client.print(',');
            client.print( common_color[1], DEC );
            client.print(',');
            client.print( common_color[2], DEC );

            //Send current kn(O)ck value//
          } 
          else if(buffer[5] == 'o') {

            http_response(client);
            client.print( knock, DEC );

            //Catch-all, Invalid entry//   
          } 
          else {
            //Standard output
            //http_response(client);
            //client.println();
            //client.print("IE");
          }

          count = 0;
          break;
        }

        if (c == '\n') {
          // you're starting a new line
          currentLineIsBlank = true;
        } 
        else if (c != '\r') {
          // you've gotten a character on the current line
          currentLineIsBlank = false;
        }
      }
      count++;
    }
    // give the web browser time to receive the data
    delay(1);
    // close the connection:
    client.stop();
  }

  //delay(2);
  //Serial.println(availableMemory());
}
